
from platform import system

import logging

import serial.rs485
import serial.tools.list_ports

from .base_driver import BaseDriver

_logger = logging.getLogger(__name__)

try:
    import termios

    SerialError = termios.error
except ImportError:
    SerialError = None

if system() == 'Windows':
    from Emulator.TCPClient import TCPClient
    from socket import error as socket_error


class PySerialRS485Driver(BaseDriver):
    def __init__(self, port='/dev/ttyO4', baud=57600, delay_before_rx=0.01, delay_before_tx=0.01):
        if system() == 'Windows':
            self.emulatorClient = TCPClient(44444)
            self.sendBuffer = ''
            self.receiveBuffer = None
            self.receiveData = None
            _logger.debug("initializing")
        else:
            try:
                handle = serial.rs485.RS485(port)
                handle.rs485_mode = serial.rs485.RS485Settings(delay_before_rx=delay_before_rx,
                                                               delay_before_tx=delay_before_tx)
                handle.baudrate = baud
                handle.timeout = 1
                super(PySerialRS485Driver, self).__init__(handle)
            except (OSError, serial.SerialException) as e:
                print
                print "Error opening serial device '%s':" % port
                print e
                print
                print "The following serial devices were detected:"
                print
                for (name, desc, _) in serial.tools.list_ports.comports():
                    if desc[0:4] == "ttyS":
                        continue
                    if name == desc:
                        print "\t%s" % name
                    else:
                        print "\t%s (%s)" % (name, desc)
                print
                raise SystemExit

    def _getReceiveData(self, size):
        if self.receiveData:
            response = self.receiveData[0:size]
            self.receiveData = self.receiveData[size:]
            if len(self.receiveData) == 0:
                self.recieveData = None
            return response

    def read(self, size):
        """
        Read wrapper.

        Parameters
        ----------
        size : int
          Number of bytes to read.
        """
        if system() == 'Windows':
            if self.receiveData:
                return self._getReceiveData(size)
            else:
                if self.receiveBuffer:
                    # _logger.debug('response: {}'.format(self.receiveBuffer))
                    result, data = self.receiveBuffer.split(' ')
                    if result == 'ack':
                        self.receiveData = ''
                        index = 0
                        while index < len(data):
                            self.receiveData = self.receiveData + chr(int(data[index:index+2], 16))
                            index = index + 2
                        self.receiveBuffer = None
                        return self._getReceiveData(size)
                    else:
                        return None
                else:
                    return None
        else:
            try:
                return self.handle.read(size)
            except (OSError, serial.SerialException):
                _logger.debug("Power Section disconnected error on read")
                raise IOError

    def write(self, s):
        """
        Write wrapper.

        Parameters
        ----------
        s : bytes
          Bytes to write
        """
        if system() == 'Windows':
            self.sendBuffer = ''
            index = 0
            while index < len(s):
                self.sendBuffer = self.sendBuffer + format(ord(s[index]), '02x')
                index = index + 1
            try:
                # _logger.debug('pwr unit request: {}'.format(self.sendBuffer))
                self.receiveBuffer = self.emulatorClient.transmit('req PWU ' + self.sendBuffer)
                # _logger.debug('recieved {}'.format(self.receiveBuffer))
                if (self.receiveBuffer is not None) and (self.receiveBuffer != []):
                    return len(self.receiveBuffer)
                else:
                    return 0
            except TypeError:
                return 0
            except socket_error:
                return 0
        else:
            try:
                return self.handle.write(s)
            except (OSError, serial.SerialException,
                    serial.writeTimeoutError) as e:
                if e == serial.writeTimeoutError:
                    _logger.debug("psbp pyserialrs485_driver: writeTimeoutError")
                    return 0
                else:
                    _logger.debug("Power Section disconnected error on write")
                    raise IOError

    def __enter__(self):
        if system() == 'Windows':
            return self
        else:
            self.flush()
            return self

    def __exit__(self, *args):
        if system() == 'Windows':
            pass
        else:
            try:
                self.flush()
                self.close()
            except (OSError, SerialError, serial.SerialException):
                pass
